/*****************************************************************************************************

	MCD03.cpp

	MCD03 basic control Ver.1.0
	 
	
													2010.7.2 M.Utsumi@Arcdevice 

******************************************************************************************************/

#include"stdafx.h"
#include<stdio.h>
#include<math.h>
#include"MCD03.h"


MCD03::MCD03()
{
	wTxBufCount = 1;	// R}h̕
	wRxBufCount = 0;
	bStatus = MCD03_NORMAL;
}

MCD03::~MCD03()
{
	if(	hUSB )	FT_Close( hUSB );
}

BOOL MCD03::DeviceOpen( DWORD dwIndex, BYTE bTargetID, DWORD dwAttrsAndFlags )
{
		DWORD dwLocID;
		BYTE b;

		ftStatus = FT_ListDevices( (PVOID)dwIndex, &dwLocID, FT_LIST_BY_INDEX|FT_OPEN_BY_LOCATION);
		if( ftStatus != FT_OK ){
			bStatus = MCD03_HOST_USB_ERROR;
			sprintf_s( Error, "Host USB Error in FT_ListDevices. ftStatus %d \r\n", ftStatus );
			return( FALSE );
		}
	
		hUSB = FT_W32_CreateFile( (LPCTSTR)dwLocID,GENERIC_READ|GENERIC_WRITE,0,0,
								OPEN_EXISTING,
								dwAttrsAndFlags,
								0);
		if (hUSB == INVALID_HANDLE_VALUE) return( FALSE );
		
		ftStatus = FT_SetTimeouts( hUSB, 5000, 5000 );
		if( ftStatus != FT_OK ){
			bStatus = MCD03_HOST_USB_ERROR;
			sprintf_s( Error, "Host USB Error in FT_SetTimeouts. ftStatus %d \r\n", ftStatus );
			return( FALSE );
		}
	
		if( bTargetID == MCD03_MASTER_ID )	return( TRUE );
		
		GetID( &b );
		if( bTargetID !=  b ){
			FT_W32_CloseHandle( hUSB );
			return( FALSE );
		}

		return( TRUE );
}

BOOL MCD03::Open( BYTE bID )
{
	FT_STATUS ftStatus;
	DWORD dwNumDevice, dwIndex, dwAttrsAndFlags;

	ftStatus = FT_ListDevices( &dwNumDevice, NULL, FT_LIST_NUMBER_ONLY );
	if( ftStatus != FT_OK ){
		bStatus = MCD03_HOST_USB_ERROR;
		sprintf_s( Error, "Host USB Error in FT_ListDevices. ftStatus %d \r\n", ftStatus );
		return( FALSE );
	}
	
	if( dwNumDevice == 0 ){
		sprintf_s(  Error, "No FT device found\r\n");
		return( FALSE );
	}

	dwAttrsAndFlags = FILE_ATTRIBUTE_NORMAL | FT_OPEN_BY_LOCATION;
	
	for( dwIndex=0; dwIndex<dwNumDevice; dwIndex++){
		if( DeviceOpen( dwIndex, bID, dwAttrsAndFlags ) ){
			ftStatus = FT_SetLatencyTimer( hUSB, 0 );
			if( ftStatus != FT_OK ){
				bStatus = MCD03_HOST_USB_ERROR;
				sprintf_s( Error, "Host USB Error in FT_SetLatencyTimer. ftStatus %d \r\n", ftStatus );
				return( FALSE );
			}

			if( !Transceive( MCD03_OPEN, 0 )) return( FALSE );

			return( TRUE );
		}
	}
	sprintf_s( Error, "Target ID device not found\r\n" );

	return( FALSE );
}

BOOL MCD03::Close( void )
{
	if( !Transceive( MCD03_CLOSE, 0 )) return( FALSE );

	FT_Close( hUSB );
	hUSB = NULL;
	return( TRUE );
}

void MCD03::ProcDeviceError( void  )
{
	switch( bStatus ){
	case MCD03_UNDEFINED_COMMAND:
		sprintf_s( Error, "Undefined command\r\n");
		break;
	case MCD03_CTRL_TIMEOUT:
		sprintf_s( Error, "Control Timeout\r\n");
		break;
	case MCD03_TX_TIMEOUT:
		sprintf_s( Error, "Device TX Timeout\r\n");
		break;
	case MCD03_RX_TIMEOUT:
		sprintf_s( Error, "Device RX Timeout\r\n");
		break;
	default :
		sprintf_s( Error, "Undefined Status Received\r\n");
		bStatus = MCD03_UNDEFINED_STATUS;
		break;
	}
}


BOOL MCD03::Transceive( BYTE bCommand, WORD wRxDataSize )
{
	WORD wRxSize;
	DWORD dwBytesWritten;
	DWORD dwBytesReceived;
	
	bTxBuffer[0] = bCommand;

	ftStatus = FT_Write(hUSB, bTxBuffer, wTxBufCount, &dwBytesWritten);
	if( ftStatus !=  FT_OK ){
		bStatus = MCD03_HOST_USB_ERROR;
		sprintf_s( Error, "Host USB Error! ftStatus %d \r\n", ftStatus );
		return( FALSE );
	}
	if( dwBytesWritten != wTxBufCount ){
		bStatus = MCD03_HOST_TX_TIMEOUT;
		sprintf_s( Error, "Host TX Timeout Error!\r\n" );
		return( FALSE );
	}
	
	wTxBufCount = 1;	// R}h̕
	wRxBufCount = 0;
	wRxSize = wRxDataSize + 1; // MTCY̓Xe[^X̕𑫂.

	ftStatus = FT_Read( hUSB, bRxBuffer, wRxSize, &dwBytesReceived);
	if( ftStatus != FT_OK ){
		bStatus = MCD03_HOST_USB_ERROR;
		sprintf_s( Error, "Host USB Error! ftStatus %d \r\n", ftStatus );
		return( FALSE );
	}

	if( dwBytesReceived == 0 ){
		bStatus = MCD03_HOST_RX_TIMEOUT;
		sprintf_s( Error, "Host RX Timeout Error!\r\n" );
		return( FALSE );
	}

	ReadBuf( &bStatus, 1 );
	if( bStatus!=MCD03_NORMAL ){
		ProcDeviceError();
		return( FALSE );
	}

	if( dwBytesReceived != wRxSize ){
		bStatus = MCD03_HOST_RX_TIMEOUT;
		sprintf_s( Error, "Host RX Timeout Error!\r\n" );
		return( FALSE );
	}
	
	return( TRUE );
}


void MCD03::WriteBuf( void* pBuffer, WORD wSize )
{
	 memcpy( &bTxBuffer[wTxBufCount], pBuffer, wSize );
	 wTxBufCount+=wSize;
}

void MCD03::ReadBuf( void* pBuffer, WORD wSize )
{
	 memcpy( pBuffer, &bRxBuffer[wRxBufCount], wSize );
	 wRxBufCount+=wSize;
}


BOOL MCD03::Reset( void )
{
	ftStatus = FT_ResetPort( hUSB );
	if( ftStatus !=  FT_OK ){
		bStatus = MCD03_HOST_USB_ERROR;
		sprintf_s( Error, "USB Error! ftStatus %d \r\n", ftStatus );
		return( FALSE );
	}
		
	Sleep(3000);	//foCXA܂ŎԒ
	return( TRUE );
}

BOOL MCD03::SetID( BYTE bID )
{
	WriteBuf( &bID, 1 );
	
	if( !Transceive( MCD03_SET_ID, 0 )) return( FALSE );

	return( TRUE );
}

BOOL MCD03::GetID( BYTE* pbID )
{
	if( !Transceive( MCD03_GET_ID, 1 )) return( FALSE );

	ReadBuf( pbID, 1 );

	return( TRUE );
}

BOOL MCD03::GetVersion( char Version[64] )
{
	if( !Transceive( MCD03_GET_VERSION, 64 )) return( FALSE );

	ReadBuf( Version, 64 );

	return( TRUE );
}

BOOL MCD03::Blink( BYTE bNum )
{
	WriteBuf( &bNum, 1) ;

	if( !Transceive( MCD03_BLINK, 0 )) return( FALSE );

	return( TRUE );
}

BOOL MCD03::Stop( void )
{

	if( !Transceive( MCD03_STOP, 0 )) return( FALSE );

	return( TRUE );
}

BOOL MCD03::SetParam( PMCD03_PARAM pParam )
{
	DWORD dwTimeout;

	WriteBuf( pParam, sizeof( MCD03_PARAM ) );

	if( !Transceive( MCD03_SET_PARAM, 0 )) return( FALSE );

	dwTimeout = (DWORD)pParam->fCommTimeout_ms;

	ftStatus = FT_SetTimeouts( hUSB, dwTimeout, dwTimeout );
	if (ftStatus != FT_OK){
		bStatus = MCD03_HOST_USB_ERROR;
		sprintf_s( Error, "USB Error! FT_SetTimeouts failed\n");
		return( FALSE ); 
	}

	return( TRUE );
}

BOOL MCD03::GetParam( PMCD03_PARAM pParam )
{
	if( !Transceive( MCD03_GET_PARAM, sizeof(MCD03_PARAM ) )) return( FALSE );

	ReadBuf( pParam, sizeof(MCD03_PARAM ) );

	return( TRUE );
}

BOOL MCD03::GetData( void )
{
	if( !Transceive( MCD03_GET_DATA, sizeof(float)*3+2 )) return( FALSE );

	ReadBuf( &fPosition, sizeof(float ) );
	ReadBuf( &fVelocity, sizeof(float ) );
	ReadBuf( &fTorque,   sizeof(float ) );
	ReadBuf( &wDiPort,   sizeof(WORD  ) );

	return( TRUE );
}

BOOL MCD03::ClearData( void )
{
	if( !Transceive( MCD03_CLEAR_DATA, 0 )) return( FALSE );

	return( TRUE );
}

BOOL MCD03::SimMotor( float fMaxVelocity )
{
	WriteBuf( &fMaxVelocity, sizeof(float) );
	
	if( !Transceive( MCD03_SIM_MOTOR, 0 )) return( FALSE );

	return( TRUE );
}

BOOL MCD03::SetPosition( float fRefPosition, float fVelocityLimit, float fTorqueLimit )
{
	WriteBuf( &fRefPosition, sizeof( float ) );
	WriteBuf( &fVelocityLimit, sizeof( float ) );
	WriteBuf( &fTorqueLimit, sizeof( float ) );

	if( !Transceive( MCD03_SET_POSITION, sizeof(float)*3+2 )) return( FALSE );

	ReadBuf( &fPosition, sizeof(float ) );
	ReadBuf( &fVelocity, sizeof(float ) );
	ReadBuf( &fTorque,   sizeof(float ) );
	ReadBuf( &wDiPort,   sizeof(WORD  ) );

	return( TRUE );
}

BOOL MCD03::SetVelocity( float fRefVelocity, float fTorqueLimit )
{
	WriteBuf( &fRefVelocity, sizeof( float ) );
	WriteBuf( &fTorqueLimit, sizeof( float ) );

	if( !Transceive( MCD03_SET_VELOCITY, sizeof(float)*3+2 )) return( FALSE );

	ReadBuf( &fPosition, sizeof(float ) );
	ReadBuf( &fVelocity, sizeof(float ) );
	ReadBuf( &fTorque,   sizeof(float ) );
	ReadBuf( &wDiPort,   sizeof(WORD  ) );

	return( TRUE );
}

BOOL MCD03::SetTorque( float fRefTorque  )
{
	WriteBuf( &fRefTorque, sizeof( float ) );

	if( !Transceive( MCD03_SET_TORQUE, sizeof(float)*3+2 )) return( FALSE );

	ReadBuf( &fPosition, sizeof(float ) );
	ReadBuf( &fVelocity, sizeof(float ) );
	ReadBuf( &fTorque,   sizeof(float ) );
	ReadBuf( &wDiPort,   sizeof(WORD  ) );

	return( TRUE );
}

BOOL MCD03::SetDuty( SHORT sDuty  )
{
	WriteBuf( &sDuty, sizeof( SHORT ) );

	if( !Transceive( MCD03_SET_DUTY, sizeof(float)*3+2 )) return( FALSE );

	ReadBuf( &fPosition, sizeof(float ) );
	ReadBuf( &fVelocity, sizeof(float ) );
	ReadBuf( &fTorque,   sizeof(float ) );
	ReadBuf( &wDiPort,   sizeof(WORD  ) );

	return( TRUE );
}






