/*****************************************************************************************************

	MCD03.h

	MCD03 basic control Ver.1.0
	 
	
													2010.7.2 M.Utsumi@Arcdevice 

******************************************************************************************************/

#ifndef MCD03_H
#define MCD03_H

#include<windows.h>
#include"ftd2xx.h"

#ifndef _CM_PHYSICS_PARAM
#define _CM_PHYSICS_PARAM

#define  M_PI   3.141593f
#define  M_hPI   1.570796f
#define  M_2PI   6.283185f
 
#define DEGtoRAD 0.01745329252f 
#define RADtoDEG 57.29577951f

#define RADSECtoRPM 9.549296586 
#define RPMtoRADSEC 0.1047197551f
#define kgfcmToNm		0.09807f	
#define NmTokgfcm		10.1968f

#endif

#define MCD03_MASTER_ID		0xFF

// Command

#define MCD03_OPEN 				0
#define MCD03_CLOSE 			1
#define MCD03_RESET 			2
#define MCD03_SET_ID 			3
#define MCD03_GET_ID	 		4
#define MCD03_GET_VERSION 		5
#define MCD03_BLINK 			6

#define MCD03_SET_PARAM 		10
#define MCD03_GET_PARAM 		11
#define MCD03_CLEAR_DATA	 	12
#define MCD03_GET_DATA	 		13
#define MCD03_SIM_MOTOR 		14

#define MCD03_STOP 				20
#define MCD03_SET_POSITION		21
#define MCD03_SET_VELOCITY 		22
#define MCD03_SET_TORQUE 		23
#define MCD03_SET_DUTY 			24


// Status

#define MCD03_NORMAL				100

#define MCD03_UNDEFINED_COMMAND		101
#define MCD03_CTRL_TIMEOUT			102
#define MCD03_TX_TIMEOUT			103
#define MCD03_RX_TIMEOUT			104

#define MCD03_HOST_USB_ERROR		110
#define MCD03_HOST_TX_TIMEOUT		111		
#define MCD03_HOST_RX_TIMEOUT		112		
#define MCD03_UNDEFINED_STATUS		113


// ʒuZT̐ݒ
#define MCD03_POS_SENS_POTENTIO		0
#define MCD03_POS_SENS_ENCODER		1

typedef struct _MCD03_PARAM
{
	float dt_ms;			// [ms]
	float fCtrlTimeout_ms;	// ^CAEg[ms]
	float fCommTimeout_ms;	// ʐM^CAEg[ms]
	float kp;				// ʒuPQC
	float kd;				// ʒuDQC
	float kv;				// xPQC
	float ki;				// xIQC
	float kt;				// gNIQC
	float fMinEn;			// ʒu̍ŏ덷[rad]or[m]or[mm]
	float fPosLimit_h;		// ʒu~bgHigh[rad]or[m]or[mm]
	float fPosLimit_l;		// ʒu~bgLow[rad]or[m]or[mm]
	float fPosOffset;		// ʒuZT̃ItZbg[rad]or[m]or[mm]
	float ken;				// GR[_pXϊW[rev/pulse]or[m/pulse]
	float kpt;				// |eV[^ϊW[rad/ad]or[m/ad]
	float ktq;				// gNϊW[mNm/ad]or[A/ad]

} MCD03_PARAM, *PMCD03_PARAM;


class MCD03
{

public:
	MCD03();
	~MCD03();

	BOOL Open( BYTE bID );
	BOOL Close( void );
	BOOL Reset( void );
	BOOL SetID( BYTE bID );
	BOOL GetID( BYTE* pbID );
	BOOL GetVersion( char Version[64] );
	BOOL Blink( BYTE bNum );		

	BOOL SetParam( PMCD03_PARAM pParam );
	BOOL GetParam( PMCD03_PARAM pParam );
	BOOL ClearData( void  );
	BOOL GetData( void );
	BOOL SimMotor( float fMaxVelocity );

	BOOL Stop( void );
	BOOL SetPosition( float fRefPosition, float fVerocityLimit, float fTorqueLimit );
	BOOL SetVelocity( float fRefVerocity, float fTorqueLimit );
	BOOL SetTorque( float fRefTorque  );
	BOOL SetDuty( SHORT sDuty );

	BYTE bStatus;
	char Error[256];

	float fPosition;
	float fVelocity;
	float fTorque;
	WORD wDiPort;

private:
	BOOL DeviceOpen( DWORD dwIndex, BYTE bTargetID, DWORD dwAttrsAndFlags );
	void ProcDeviceError( void );
	BOOL Transceive( BYTE bCommand, WORD wRxDataSize );
	void WriteBuf( void* pBuffer, WORD wSize );
	void ReadBuf( void* pBuffer, WORD wSize );

	FT_HANDLE hUSB;
	FT_STATUS ftStatus;
	
	DWORD dwTxBytes; 
	DWORD dwRxBytes; 
	BYTE  bTxBuffer[256];
	BYTE  bRxBuffer[256];
	WORD  wTxBufCount;
	WORD  wRxBufCount;

};


#endif
